#include "config.h"
#include "macros.h"
#include "readconf.h"
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>

struct confstruct conf;


void config_free()
{
	struct confshare *s = conf.sharelist;
	struct confshare *next;

	context;

	while(s)
	{
		next = s->next;
		ag_free(s);
		s = next;
	}
}


/* expands ~ into /home/whatever/ */
unsigned char *expand(unsigned char *path)
{
	static unsigned char npath[MAX_PATH];
	char *home;

	if(*path != '~')
		return path;

	home = getenv("HOME");
	snprintf(npath, MAX_PATH, "%s/%s", home?home:".", path+1);

	return npath;
}


int config_read()
{
	FILE *fd;
	unsigned char str[256], cmd[32], opt[224], act[224];
	int line=0,i;
	struct confshare *s=NULL;

	memset(&conf, 0, sizeof(conf));

	fd = fopen(etc_file("config"), "r");
	if(!fd) {
		NOTICE("Unable to open '%s': %s", etc_file("config"), ERRMSG());
		return 0;
	}

	while(fgets(str, sizeof(str), fd)) {
		line++;
		if(str[0] == 0 || str[0] == '#' || str[0] == ';')
			continue;

		i = strlen(str);
		if(i == 1) continue;			/* skip empty lines */
		if(i > 0 && str[i-1] == '\n')	/* strip trailing newline unsigned char */
			str[i-1] = 0;

		memset(&opt, 0, sizeof(opt));
		if(sscanf(str, "%32s %222c", cmd, opt) != 2) {
			NOTICE("Parse error on line %d in config", line);
			return 0;
		}

		if(!strcasecmp(cmd, "user")) {
			ag_strncpy(conf.username, opt, sizeof(conf.username));

		} else if(!strcasecmp(cmd, "pass")) {
			ag_strncpy(conf.password, opt, sizeof(conf.password));

		} else if(!strcasecmp(cmd, "share")) {
			if(s)
			{
				s->next = ag_malloc(sizeof(struct confshare));
				s = s->next;
			}
			else
			{
				s = conf.sharelist = ag_malloc(sizeof(struct confshare));
			}
			s->next = NULL;

			/* last 2 bytes in opt are \0 so this is safe */
			if(opt[strlen(opt)-1] != '/')
				opt[strlen(opt)] = '/';

			ag_strncpy(s->path, expand(opt), MAX_PATH);

		} else if(!strcasecmp(cmd, "download_dir")) {
			ag_strncpy(conf.download_dir, expand(opt), MAX_PATH);

		} else if(!strcasecmp(cmd, "event")) {
			memset(&act, 0, sizeof(act));
			if(sscanf(opt, "%32s %223c", cmd, act) != 2) {
				NOTICE("Parse error on event cmd, line %d in config", line);
				return 0;
			}

			event_register(cmd, expand(act));

		} else if(!strcasecmp(cmd, "hostname")) {
			ag_strncpy(conf.hostname, opt, sizeof(conf.hostname));

		} else if(!strcasecmp(cmd, "ip")) {
			ag_strncpy(conf.ip, opt, sizeof(conf.ip));

		} else {
			NOTICE("Unknown option on line %d in fags.conf", line);
		}
	}

	fclose(fd);


	if(!conf.username[0] || !conf.password[0])
	{
		NOTICE("No username or password set!");
		return 0;
	}

	if(!conf.sharelist)
	{
		NOTICE("No share directory set!");
		return 0;
	}

	if(!conf.download_dir[0])
	{
		/* default to "." */
		conf.download_dir[0] = '.';
		conf.download_dir[1] = 0;
	}

	return 1;
}



int config_setup_write(unsigned char *file, int exec, unsigned char *cont)
{
	FILE *fd;
	int len;

	if(!file_exists(file))
	{
		fd = fopen(file, "w");
		if(!fd)
		{
			printf("Can't open '%s': %s\n", file, ERRMSG());
			return 1;
		}

		len = strlen(cont);
		if(fwrite(cont, 1, len, fd) != len)
		{
			printf("Error writing to '%s': %s\n", file, ERRMSG());
			fclose(fd);
			return 1;
		}
		fclose(fd);
		if(exec)
			chmod(file, S_IRUSR|S_IWUSR|S_IXUSR);
		else
			chmod(file, S_IRUSR|S_IWUSR);
		printf("Wrote %s...\n", file);
		return 2;
	}

	return 0;
}


int config_setup_homedir()
{
	int err,ret=0;

	/* create ~/.fags/ */
	if(mkdir(etc_file(""), S_IRUSR|S_IWUSR|S_IXUSR))
	{
		if(errno != EEXIST)
		{
			printf("Can't create directory '%s': %s\n", etc_file(""), ERRMSG());
			return 0;
		}
	} else
		printf("Created directory %s...\n", etc_file(""));

	/* create ~/.fags/scripts/ */
	if(mkdir(etc_file("scripts"), S_IRUSR|S_IWUSR|S_IXUSR))
	{
		if(errno != EEXIST)
		{
			printf("Can't create directory '%s': %s\n", etc_file(""), ERRMSG());
			return 0;
		}
	} else 
		printf("Created directory %s...\n", etc_file("scripts"));


	err = config_setup_write(etc_file("config"), 0,
		"# Login details\n"
		"user            username\n"
		"pass            password\n"
		"\n"
		"# Shared directories (at least 1 is required)\n"
		"share			/pub/music/\n"
		"#share			/pub/music2/\n"
		"#share			/mnt/mp3/\n"
		"\n"
		"# Where to put downloads (default is \".\")\n"
		"#download_dir		/pub/music/download\n"
		"\n"
		"# Event scripts (multiple scripts for the same event is fine)\n"
		"#event	connect			~/.fags/scripts/connect.sh\n"
		"#event	disconnect		~/.fags/scripts/disconnect.sh\n"
		"#event	upload_start	~/.fags/scripts/upload_start.sh\n"
		"#event	upload_end		~/.fags/scripts/upload_end.sh\n"
		"#event	download_start	~/.fags/scripts/download_start.sh\n"
		"#event	download_end	~/.fags/scripts/download_end.sh\n"
		"#event	delete			~/.fags/scripts/delete.sh\n"
		"#event	play_file		~/.fags/scripts/playfile.sh\n"
		"#event	play_file		~/.fags/scripts/playfile2.sh\n"
		"#event	play_file		~/.fags/scripts/playfile3.sh\n"
		"\n"
		"# In case hostname and IP detection fails, fill these in\n"
		"#hostname		bogbrush\n"
		"#ip			1.2.3.4\n"
		"\n"
		/*
		"\n"
		"\n"
		"# Share downloaded files? (default is no)\n"
		"#\n"
		"# download_dir must be located under one of\n"
		"# your shared directories for this to work\n"
		"#share_downloads		no\n"
		"\n"*/
	);
	if(err == 2)
		ret = 1;

	err = config_setup_write(etc_file("scripts/connect.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# connect.sh - args: <server-ip> <server-port>\n"
		"logger -t fags \"Connected to server $1:$2.\"\n"
	);
	if(err == 1)
		ret = 2;

	err = config_setup_write(etc_file("scripts/disconnect.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# disconnect.sh - args: none\n"
		"logger -t fags \"Disconnected from server.\"\n"
	);
	if(err == 1)
		ret = 2;


	err = config_setup_write(etc_file("scripts/download_start.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# download_start.sh - args: <path> <file> <ip>\n"
		"logger -t fags \"Downloading '$2' from $3.\"\n"
	);
	if(err == 1)
		ret = 2;

	err = config_setup_write(etc_file("scripts/download_end.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# download_end.sh - args: <path> <file> <avg speed>\n"
		"logger -t fags \"Download of '$2' completed. Average speed was $3KB/s.\"\n"
	);
	if(err == 1)
		ret = 2;


	err = config_setup_write(etc_file("scripts/upload_start.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# upload_start.sh - args: <path> <file> <ip>\n"
		"logger -t fags \"Uploading '$2' to $3.\"\n"
	);
	if(err == 1)
		ret = 2;

	err = config_setup_write(etc_file("scripts/upload_end.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# upload_end.sh - args: <path> <file> <avg speed>\n"
		"logger -t fags \"Uploaded '$2'. Average speed was $3KB/s.\"\n"
	);
	if(err == 1)
		ret = 2;

	err = config_setup_write(etc_file("scripts/delete.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# delete.sh - args: <path> <file>\n"
		"echo \"Server wants us to delete '$2/$3'\"\n"
	);
	if(err == 1)
		ret = 2;

	err = config_setup_write(etc_file("scripts/playfile.sh"), 1,
		"#!/bin/sh\n"
		"\n"
		"# playfile.sh - args: <op (play or stop)> <path> <file>\n"
		"echo \"Server wants us to $1 '$2/$3'\"\n"
	);
	if(err == 1)
		ret = 2;

	return ret;
}


